# from affiliation_parser import parse_affil
from pubmed_extract.parse import parse_affil
from pubmed_extract.extraction import my_function
from tabulate import tabulate


def add_list_record():
    # creating an empty listd
    lst = []

    # number of elements as input
    n = int(input("Enter number of elements : "))

    # iterating till the range
    for i in range(0, n):
        ele = int(input())
        lst.append(ele)  # adding the element

    a_list = [','.join(map('{0}'.format, lst))]
    print(a_list)
    # print(type(a_list))
    print("Please wait !!")
    x = my_function(a_list)
    return x
    exit()


def add_variable():
    one_pmid = input("Enter PMID: ")
    a_list = [one_pmid]
    print("Please wait !!")
    x = my_function(a_list)
    return x


def add_sting():
    keyword = input("Enter keyword : ")
    a_list = [keyword]
    print("Please wait !! \n the count is")
    x = my_function(a_list)
    return x


def affilin_parser():
    affiliation_details = input("Enter affiliation details : ")
    x = parse_affil(affiliation_details)
    print("\n", x, "\n")
    return x


print(" For keyword entry select--> a \n For single pmid select--> b  \n For adding a list of pmid select-->c "
      "\n For affiliation_parser-->d \n "
      "Press any key to exit \n")
menuchoices = {'a': add_sting, 'b': add_variable, 'c': add_list_record, 'd': affilin_parser}

try:
    ret = menuchoices[input()]()
except:
    print("Wrong_choice")
    exit()

# alist=['23184261', '23184272', '23184263']



# keyword list

DEPARMENT = frozenset(
    [
        "laboratorio",
        "laboratories",
        "laboratory",
        "laboratoire",
        "institute",
        "academic",
        "department",
        "division",
        "faculty of ",
        "genomics center",
        "research station",
    ]
)

INSTITUTE = frozenset(
    [
        "college",
        "university",
        "universitat",
        "universite",
        "unversiteit",
        "universita",
        "universidad",
        "universiti",
        "hospital",
        "hopitaux de",
        "unidade de",
        "ha'pital",
        "istituti",
        "istituto",
        "institucio",
        "institut",
        "medical center",
        " pharma",
        "riuniti",
        "clinic",
        " school of medicine",
        "karolinska sjukhuset",
        "national institutes of health",
        "cancer center",
        "bioscience institute",
        "national institute for",
        "national center for ",
        "national centre for",
        "unilever research",
        "national cardiovascular center",
        "centro operativo",
        "animal research centre",
        "nutrition research center",
        "national perinatal epidemiology unit",
        "tanabe seiyaku",
        "animal health trust",
        "marine biological laboratory",
        " medical school",
        " research laboratories",
        "baxter diagnostics",
        "inserm",
        "sylvius laboratory",
        "broad institute",
        " inra",
        "/inra",
        "health chemical laboratory",
        "genecor, inc",
        "infirmary",
        "national center for health",
        "john innes centre",
        "chru de la timone",
        "chu de bordeaux",
        "ecole nationale ",
        "cape technologies",
        "national chemical laboratory",
        " national laboratory",
        "department of research and development",
        "academy of sciences",
        "centre chirurgical de la porte",
        "international centre of ",
        "lawrence berkeley laboratory",
        "albert einstein college",
        "gedeon richter ltd",
        " nih",
        "ufrgs",
        "national research centre",
        " co.",
        " ltd.",
        " ltd",
        "inc.",
        "research limited",
        "clinic college of",
        "center for",
        "research center",
        "research centre",
        "schon klinik",
        "innovaderm research",
        "novartis",
        "aquarium",
        "foundation",
        "permanente",
        "healthcare system",
        "national oncology institute",
        "global research and development",
        "health service",
        "national primate research center",
        "faculdade de ",
        " urmc",
        " pllc",
        " pgimer",
        "center for disease control",
        "london school of ",
        "ggze",
        "health service executive",
        "council for scientific",
        "cnrs",
        "eth zurich",
        "johns hopkins",
        "isconova",
        "barts health",
        "ceinge",
        "national jewish health",
        "german institute",
        "iqwig",
        "federal joint committee" "nationale contre",
        "cura villa maria",
        "centre de psychologie",
        "centro diagnostico",
        "international reference centre",
        "complesso integrato",
        "health care centre",
        "idiphim",
        "cytogenetic laboratory",
        "fondazione",
        "facebook",
        "google",
        "association for",
        " llc",
        "national museum",
        "national research council",
        "rehabilitation center",
        "rehabilitation institute",
        "oncology center",
        "cancer centre",
        "virginia tech",
        "ciberesp",
        "department of food",
        "rothamsted research",
        "evangelisches",
        "ziekenhuizen",
        "academy of ",
        "chinese national ",
        "pathology associates",
        "science magnet",
        "ucla ",
        " ucsd",
        "uc berkeley",
        "uc san diego",
        "trial group",
        "acdi",
        "specialty center",
        "agemetra",
        "national research institute",
        "diabetes center",
        "rothamsted research",
        "affichem",
        "disease association",
        "ministry of health",
        "incorporation",
        "medical research council",
        "develogen",
        "innovation campus",
        "flemish government",
        "centre d'etudes",
        "kaist",
        "epfl",
        " eth",
        "ecole normale",
        "ecole polytechnique",
        "mental health center",
        "charite centrum",
        "phc affairs",
        "afmc",
        "cdsr",
        "chu de ",
        "harvard school",
        "karnavati school",
        "academic centre for",
        "school of public health",
        "school of sport sciences",
        "medical center",
        "medical centre",
        "neocodex",
        "umc utrecht",
        "centers for disease",
        "cardiac surgery center",
        "medical city",
        "wisconsin department",
        "doctor's data",
        "drug development office",
        "research unit",
        "ecogen",
        "international corporation",
        "tourism agency",
        "naval research laboratory",
        "infection research",
        "health solutions",
        "us military",
        "us department",
        "human genome center",
        "siemens",
        "swiss institute",
        "usda",
        "marine science center",
        "u.s. geological",
        "u.s. positive",
        "u.s. Department",
        "botanical center",
        "municipal centre",
        "municipal health",
        "research council",
        "national serology",
        "national sexually",
        "d'aragona",
        "metropolitan health",
        "rosa and company",
        "laboratory of oncology",
        "oncology r&d",
        "assessment service",
        "cancer registry",
        "technology agency",
        "district health",
        "irccs",
        "pharmexa",
        "scientific service",
        "limited company",
        "health authority",
        "biodiversity center",
        "national park",
        "corporation",
        "ucl ",
        "escola nacional",
        "va health system",
        "agri-food",
        "agrotech",
        "agroforestry",
        "umr micalis",
        "allan rosenfield",
        "allan wilson",
        "allen institute",
        "ameripath",
        "biotechnologies",
        "anaerobe systems",
        "nhs trust",
    ]
)

REMOVE_INSTITUE = frozenset(
    [
        "pharmacology",
        "college of pharmacy",
        "institute of zoology",
        "institute of population",
        "institute of bioinformatics",
        "institute of plant",
        "section for ",
        "institute of clinical medicine",
        "department of clinical",
    ]
)

COUNTRY = (
    ("italy", "italia", "torino", "turin", "portici", "palermo"),
    (
        "united kingdom",
        "u.k.",
        "\buk\b",
        "uk.",
        "england",
        " uk",
        "liverpool",
        "london",
        "crumpsall",
        "leicester",
        "manchester",
        "cardiff",
        "salford",
        "bradford",
        "oxford",
        "clwyd",
        "edinburgh",
        "edinburge",
    ),
    (
        "united states of america",
        "u.s.a",
        "u. s. a.",
        "united states",
        "massachusetts",
        "boston",
        "howard university",
        "torrance",
        "san francisco",
        "duarte",
        "los alamos",
        "houston",
    ),
    (
        "germany",
        "frg",
        "brd",
        "f.r.g.",
        "deutschland",
        "engelskirchen",
        "berlin",
        "hannover",
        "marburg",
        "mainz",
        "leipzig",
        "frankfurt",
    ),
    (
        "japan",
        "keio University",
        "jikei university",
        "shiga university",
        "jikei university",
        "niigata university",
        "sendai city",
        "shiba",
        "asahikawa",
        "tokyo",
        "yokohama",
        "osaka",
        "nagoya",
        "sapporo",
        "kobe",
        "kyoto",
        "fukuoka",
        "kawasaki",
        "saitama",
        "hiroshima",
        "sendai",
        "kitakyushu",
        "chiba",
        "sakai",
        "hamamatsu",
        "niigata",
        "shizuoka",
        "okayama",
        "asahikawa",
        "yamaguchi",
        "okayama",
        "gunma",
        "hyogo",
        "kanazawa",
        "fukui",
        "tajimi",
        "komagome",
        "akita",
        "suita",
        "kochi",
        "nara medical",
        "keio",
        "kobe university",
        "gunma",
        "gifu university",
        "toranomon",
        "bunkyo gakuin",
    ),
    ("south korea", "korea", "seoul"),
    ("russia", "moscow"),
    ("austria", "linz", "wien", " graz", "innsbruck"),
    ("israel", "jerusalem", "haifa", "tel aviv"),
    ("norway", " oslo"),
    ("finland", "helsinki"),
    ("south africa", "johannesburg"),
    ("france", "paris", "marseille", "brest", "limoges", "toulouse"),
    ("canada", "vancouver", "ontario", "ottawa", "nova scotia", "calgary", "alberta"),
    ("denmark", "copenhagen"),
    ("taiwan", "taiwan, ", "taipei", "chi-mei medical"),
    ("china", "beijing", "pr china", "hangzhou", "zhejiang", "shenyang", "shanghai"),
    ("egypt", "cairo"),
    ("poland", "gdansk", "krakow"),
    ("turkey", "istanbul", " ankara"),
    ("netherlands", "utrecht", "nijmegen", "amsterdam", "leiden"),
    ("belgium", "belgique", " namur", "bruxelles", " genk"),
    ("sweden", "karlstad", "uppsala", "stockholm"),
    ("australia", " perth", "queensland", "canberra", "melbourne"),
    ("south africa", "johannesburg", "onderstepoort", "pretoria"),
    ("hungary", "ungarn"),
    (
        " india",
        "chandigarh",
        "hyderabad",
        "delhi",
        "calcutta",
        "wardha",
        " ucms",
        "ludhiana",
        "vellore",
    ),
    ("ireland", "dublin", "belfast"),
    ("colombia", "bogota"),
    (
        "spain",
        " madrid",
        " toledo",
        " alicante",
        "zaragoza",
        "saragossa",
        "barcelona",
        "hospital vega baja",
        "valencia",
        "malaga",
    ),
    ("greece", "athens"),
    ("new zealand", "upper hutt", "auckland"),
    ("saudi arabia", "riyadh"),
    ("nigeria", "ibadan"),
    ("croatia", "yugoslavia", "belgrade"),
    ("switzerland", "basel", " geneva"),
    ("brazil", "rio de janeiro"),
    ("thailand", "bangkok"),
    ("argentina", "buenos aires"),
    ("tunisia", "tunis", "tunisie"),
    ("czech republic", " praha", "prague", "czechoslovakia"),
    ("hungary", "budapest"),
    ("zimbabwe", "bulawayo"),
    ("malaysia", "kelantan"),
    ("vietnam", "hanoi"),
    ("hong kong", "hong kong"),
    (" iran", "tehran", "shiraz", "mashhad"),
    ("brazil", "brasil", "rio de janeiro", "cordeiropolis"),
    ("mexico", "mexico city"),
    ("romania", "bucharest"),
    ("congo", "democratic republic of congo"),
    ("armenia", "yerevan"),
    ("bosnia and herzegovina", "bosnia", "herzegovina"),
    ("albania", "tirana"),
    ("tanzania", "morogoro"),
    ("sri lanka", "sri lanka"),
    ("cyprus", "nicosia"),
    ("gambia", " banjul"),
    ("lithuania", "lietuvos"),
    ("peru", " lima"),
    ("portugal", "lisboa"),
    ("philippines", "manila"),
    ("kuwait", "kuwait city"),
)

STATES = frozenset(
    [
        "Alabama",
        "Alaska",
        "Arizona",
        "Arkansas",
        "California",
        "Colorado",
        "Connecticut",
        "Delaware",
        "Florida",
        "Georgia",
        "Hawaii",
        "Idaho",
        "Illinois",
        "Indiana",
        "Iowa",
        "Kansas",
        "Kentucky",
        "Louisiana",
        "Maine",
        "Maryland",
        "Massachusetts",
        "Michigan",
        "Minnesota",
        "Mississippi",
        "Missouri",
        "Montana",
        "Nebraska",
        "Nevada",
        "New Hampshire",
        "New Jersey",
        "New Mexico",
        "New York",
        "North Carolina",
        "North Dakota",
        "Ohio",
        "Oklahoma",
        "Oregon",
        "Pennsylvania",
        "Rhode Island",
        "South Carolina",
        "South Dakota",
        "Tennessee",
        "Texas",
        "Utah",
        "Vermont",
        "Virginia",
        "Washington",
        "West Virginia",
        "Wisconsin",
        "Wyoming",
        "Washington DC",
        " AL",
        " AK",
        " AZ",
        " AR",
        " CA",
        " CO",
        " CT",
        " DE",
        " FL",
        " GA ",
        " GA,",
        " HI",
        " ID",
        " IL",
        " IN",
        " IA",
        " KS",
        " KY",
        " LA",
        " ME",
        " MD",
        " MA,",
        " MA ",
        " MI",
        " MN",
        " MS",
        " MO",
        " MT",
        " NE",
        " NV",
        " NH",
        " NJ",
        " NM",
        " NY",
        " NC",
        " ND",
        " OH",
        " OK",
        " OR",
        " PA",
        " RI",
        " SC",
        " SD",
        " TN",
        " TX",
        " UT",
        " VT",
        " VA",
        " WA",
        " WV",
        " WI",
        " WY",
        " DC",
    ]
)

# full name and abbreviation
UNIVERSITY_ABBR = (
    ("university of california los angeles", "UCLA", "UC Los Angeles"),
    ("university of california san diego", "UCSD", "UC San Diego"),
    ("california institute of technology", "caltech"),
    ("city college of new york", "CCNY"),
    ("carnegie mellon university", "CMU"),
    ("georgia institute of technology", "georgia tech"),
    ("pennsylvania state university", "penn state"),
    ("university of massachusetts amherst", "UMass"),
    ("city university of new york", " cuny"),
    ("havard university school of", "harvard school of"),
)

# use to concat university with string
UNIVERSITY_MULTIPLE_CAMPUS = (
    (
        "university of california",
        "berkeley",
        "los angeles",
        "davis",
        "davis medical center",
        "irvine",
        "irvine medical center",
        "san francisco",
        "santa barbara",
        "riverside",
    ),
    (
        "university of illinois urbana champaign",
        "urbana champaign",
        "urbana",
        "champaign",
    ),
    ("university of illinois", "urbana champaign", "chicago", "springfield", "peoria"),
    ("university of pittsburgh", "medical center"),
    ("university of michigan", "ann arbor", "dearborn", "health system", "flint"),
    (
        "university of minnesota",
        "duluth",
        "morris",
        "medical Ccnter",
        "rochester",
        "crookston",
    ),
)



import re
import string
from unidecode import unidecode
import numpy as np
from .keywords import *


def replace_institution_abbr(affil_text: str):
    """
    Replace abbreviation with full institution string
    """
    for university_list in UNIVERSITY_ABBR:
        for university in university_list:
            if university in affil_text:
                affil_text = re.sub(university, university_list[0], affil_text)
                return affil_text
    return affil_text


def append_institution_city(affil: str, location: str):
    """
    Append city to university that has multiple campuses if exist
    """
    for university_list in UNIVERSITY_MULTIPLE_CAMPUS:
        if university_list[0] in affil.lower():
            for city in university_list[1::]:
                if city in location.lower() and not city in affil.lower():
                    affil = affil + " " + city
                    return affil
    return affil


def clean_text(affil_text: str):
    """
    Given affiliation text with abbreviation, clean that text
    """
    affil_text = affil_text.strip()
    affil_text = re.sub("\t", " ", affil_text)
    affil_text = re.sub("Dept. ", "Department ", affil_text)
    affil_text = re.sub("Surg. ", "Sugery ", affil_text)
    affil_text = re.sub("Univ. ", "University ", affil_text)
    affil_text = affil_text[2:] if affil_text.startswith("2 ") else affil_text
    affil_text = affil_text[3:] if affil_text.startswith("2. ") else affil_text
    affil_text = re.sub(r"\*", " ", affil_text)
    affil_text = re.sub(";", "", affil_text)
    affil_text = re.sub("E-mail:", "", affil_text)
    affil_text = re.sub("email:", "", affil_text)
    affil_text = re.sub("P.O. Box", "", affil_text)
    affil_text = replace_institution_abbr(affil_text)
    return affil_text.strip()


def find_country(location: str):
    """
    Find country from string
    """
    location_lower = location.lower()
    for country in COUNTRY:
        for c in country:
            if c in location_lower:
                return country[0]
    return ""


def check_country(affil_text: str):
    """
    Check if any states string from USA or UK
    """
    for country in ["UK"]:
        if country in affil_text:
            return "united kingdom"
    for state in STATES:
        if state in affil_text:
            return "united states of america"
    return ""


def parse_email(affil_text: str):
    """Find email from given string"""
    match = re.search(r"[\w\.-]+@[\w\.-]+", affil_text)
    if match is not None:
        email = match.group()
        if email[-1] == ".":
            email = email[:-1]
    else:
        email = ""
    return email


def parse_zipcode(affil_text: str):
    """
    Parse zip code from given affiliation text
    """
    zip_code_group = ""
    zip_code = re.search(r"(\d{5})([-])?(\d{4})?", affil_text)
    if zip_code is None:
        zip_code = re.search(r"(\d{3})([-])?(\d{4})?", affil_text)
    else:
        zip_code = ""
    if zip_code:
        zip_code_group = zip_code.groups()
        zip_code_group = [p for p in zip_code_group if p is not None]
        zip_code_group = "".join(zip_code_group)
    return zip_code_group


def parse_location(location):
    """
    Parse location and country from affiliation string
    """
    location = re.sub(r"\.", "", location).strip()
    country = find_country(location)
    dict_location = {"location": location.strip(), "country": country.strip()}
    return dict_location


def parse_affil(affil_text):
    """
    Parse affiliation string to institution and department
    """
    affil_text = unidecode(affil_text)
    affil_text = clean_text(affil_text)
    email = parse_email(affil_text)
    zip_code = parse_zipcode(affil_text)
    affil_text = re.sub(email, "", affil_text)
    affil_text = re.sub(zip_code, "", affil_text)

    affil_list = affil_text.split(", ")
    affil = list()
    location = list()
    departments = list()

    for i, a in enumerate(affil_list):
        for ins in INSTITUTE:
            if ins in a.lower() and (not a in affil):
                affil.append(a)
                location = affil_list[i + 1::]

    # remove unwanted from affliation list and location list
    pop_index = list()
    for i, a in enumerate(affil):
        for rm in REMOVE_INSTITUE:
            if rm in a.lower() and (not "university" in a.lower()):
                pop_index.append(i)
    affil = np.delete(affil, list(set(pop_index))).tolist()

    pop_index = list()
    for i, l in enumerate(location):
        for rm in DEPARMENT:
            if rm in l.lower():
                pop_index.append(i)
    location = np.delete(location, list(set(pop_index))).tolist()

    affil = ", ".join(affil)
    location = ", ".join(location)
    if location == "":
        location = affil_text.split(", ")[-1]
    location = re.sub(r"\([^)]*\)", "", location).strip()

    for i, a in enumerate(affil_list):
        for dep in DEPARMENT:
            if dep in a.lower() and (not a in departments):
                departments.append(affil_list[i])
    department = ", ".join(departments)

    dict_location = parse_location(location)
    affil = append_institution_city(affil, dict_location["location"])

    dict_out = {
        "full_text": affil_text.strip(),
        "department": department.strip(),
        "institution": affil.strip(),
        "email": email,
        "zipcode": zip_code,
    }
    dict_out.update(dict_location)
    if dict_out["country"] == "":
        dict_out["country"] = check_country(affil_text)  # check country
    return dict_out



import json
import dotenv
import os
import sys
import requests
import xml
import pandas as pd
from tqdm import tqdm
from tabulate import tabulate
from bs4 import BeautifulSoup as bs
#from pmids_extract_solr import pmids_solr_list
from dotenv import load_dotenv

load_dotenv()
#environment variable
# API_KEY = os.getenv("API_KEY")
API_KEY = "9446fd13127051ca7343c2018204779d8007"
##
def author_details(pmid):
    keyword = pmid
    esearch = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term="
    # covid19&usehistory=true&webenv=true&retmax=10&api_key=9446fd13127051ca7343c2018204779d8007"
    search_resp = requests.get(esearch + keyword + "&usehistory=true&webenv=true" + "api_key=" + API_KEY)
    search_soup = bs(search_resp.content, features="lxml")
    if search_soup:
        # print(search_soup)
        webenv = search_soup.find("webenv")
        if webenv:
            webenv = webenv.text
            # print(webenv)
            count = search_soup.find("count").text
            print(count)
            batch_size = 2000
            batches = []
            for batch in range(0, int(count), 2001):
                efetch = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&query_key=1&webenv="
                fetch_resp = requests.get(
                    efetch + webenv + "&rettype=xml&api_key=" + API_KEY + "&retmode=xml&retmax=2000&retstart=" + str(
                        batch))
                fetch_soup = bs(fetch_resp.content, features="lxml")
                articles_list = []
                PubmedArticleSet = fetch_soup.find("pubmedarticleset")
                if PubmedArticleSet:
                    articles = PubmedArticleSet.find_all("pubmedarticle")
                    for article in articles:
                        article_dict = {}
                        title = article.find("articletitle").text
                        pmid = article.find("pmid").text
                        authors = article.find("authorlist")
                        abstract = article.find("abstract")
                        author_list = []
                        try:
                            author_l = authors.find_all("author")
                        except AttributeError:
                            break
                        for auth in author_l:
                            author_dict = {}
                            try:
                                first_name = auth.find("forename").text
                            except:
                                first_name = ""
                            try:
                                second_name = auth.find("lastname").text
                            except:
                                second_name = ""
                            try:
                                initials = auth.find("initials").text
                            except:
                                initials = ""
                            author_name = initials + " " + first_name + " " + second_name
                            try:
                                affiliation = auth.find("affiliation").text
                            except:
                                affiliation = ""
                            author_dict["author_name"] = author_name
                            author_dict["initials_name"] = initials
                            author_dict["forename_name"] = first_name
                            author_dict["last_name"] = second_name
                            author_dict["affiliation"] = affiliation
                            author_list.append(author_dict)
                        article_dict["title"] = title
                        article_dict["pmid"] = pmid
                        article_dict["author"] = author_list
                        article_dict["abstract"] = abstract
                        batches.append(article_dict)

            if batches:
                df = pd.DataFrame(batches)
                df.columns = ['title', 'pmid', 'author','abstract']
                # ref:https://stackoverflow.com/questions/63875682/pandas-dataframe-splitting-a-column-with-dict-values-into-columns
                #  explode the list
                df = df.explode('author').reset_index(drop=True)
                # now fill the NaN with an empty dict
                df.author = df.author.fillna({i: {} for i in df.index})
                # then normalize the column
                df = df.join(pd.json_normalize(df.author))
                # drop the column
                df.drop(columns=['author'], inplace=True)
                # droping duplicates
                dff = df.drop_duplicates()
                return dff


def my_function(pmid_item):
    results = pd.DataFrame()
    for x in pmid_item:
        df = author_details(x)
        results = pd.concat([results, df], axis=0).reset_index(drop=True)
    print(tabulate(results, headers='keys', tablefmt='psql'))
    # print(results)
#results.to_csv('/mnt4/aims/KOL_network/Author_name_disambiguation/author_details.csv')



#alist = pmids_solr_list

# my_function("Covid")

# author_details("Covid")


import os
import shutil
import requests
import zipfile
import io
from pathlib import Path


def download_grid_data(
    url: str = "https://figshare.com/ndownloader/files/30589659",
    file_name: str = "grid",
):
    """
    Download GRID data from https://grid.ac/downloads
    to home directory "~/.affiliation_parser/grid"
    """
    path = Path(os.getenv("~", "~/.affliation_parser")).expanduser()
    grid_path = path / f"{file_name}"
    if not grid_path.exists():
        print(f"Downloading GRID dataset to {grid_path}.")
        grid_path.mkdir(parents=True, exist_ok=True)
        # download from url
        r = requests.get(url)
        z = zipfile.ZipFile(io.BytesIO(r.content))
        z.extractall(grid_path)
    else:
        print(f"You already downloaded GRID dataset to {grid_path}.")



import os
from pathlib import Path
from typing import Optional, Union
import numpy as np
import pandas as pd

import recordlinkage
from recordlinkage.index import Full

from .utils import download_grid_data
from .parse import parse_affil

path = Path(os.getenv("~", "~/.affliation_parser")).expanduser()
grid_path = path / "grid"
if not grid_path.exists():
    download_grid_data()  # if GRID data does not exist, download it locally
grid_df = pd.read_csv(
    grid_path / "grid.csv",
    header=0,
    names=["grid_id", "institution", "city", "state", "country"],
)
grid_df[
    "location"] = grid_df.city + " " + grid_df.state  # adding location column


def match_affil(affiliation: Optional[Union[str, list]], k: int = 3):
    """
    Match a given affliation string to GRID dataset.
    Return a list of most probable match as output from GRID dataset with GRID ids.

    We perform an affiliation parser to institution, location, and country
    before matching to GRID dataset. An additional text normalization
    to the GRID format will help improve the accuracy of the matching.
    For example, US or USA -> United States.

    affiliation: str, affiliation string
    k: int, number of output matches
    """
    if isinstance(affiliation, str):
        parsed_affil = parse_affil(affiliation)
        df = pd.DataFrame([parsed_affil])
    elif isinstance(affiliation, list):
        parsed_affils = [parse_affil(a) for a in affiliation]
        df = pd.DataFrame(parsed_affils)
    else:
        return None

    indexer = recordlinkage.Index()
    indexer.add(Full())
    candidate_links = indexer.index(df, grid_df)

    # recordlinkage comparer
    compare = recordlinkage.Compare()
    compare.string("institution", "institution", method="jarowinkler")
    compare.string("location", "location", method="jarowinkler")
    compare.string("country", "country", method="jarowinkler")

    features_df = compare.compute(candidate_links, df, grid_df)
    features_df["score"] = np.average(features_df,
                                      axis=1,
                                      weights=[0.6, 0.2, 0.2])

    if isinstance(affiliation, str):
        # return a list of top-k closest match
        topk_df = (features_df[["score"]].reset_index().sort_values(
            "score", ascending=False).head(k))
        topk_df = topk_df.merge(grid_df.reset_index(),
                                left_on="level_1",
                                right_on="index").drop(
                                    labels=["level_0", "level_1", "location"],
                                    axis=1)
        return topk_df.to_dict(orient="records")

    elif isinstance(affiliation, list):
        # return a list of list of top-k closest match
        topk_list = []
        for _, topk_df in features_df.reset_index().groupby("level_0"):
            topk_df = (features_df[["score"]].reset_index().sort_values(
                "score", ascending=False).head(k))
            topk_df = topk_df.merge(grid_df.reset_index(),
                                    left_on="level_1",
                                    right_on="index").drop(labels=[
                                        "level_0", "level_1", "location"
                                    ],
                                                           axis=1)
            topk_list.append(topk_df.to_dict(orient="records"))
        return topk_list
    else:
        return None



import setuptools

setuptools.setup(name='pubmed_extract',
      version='0.1',
      description='Python Distribution Utilities',
      author='Ashutosh Kumar',
      author_email='akashutosh09@gmail.com',
      url='https://skumarashutosh.github.io/',
      packages=['pubmed_extract'],
      install_requires=[]
     )

# import json
# import dotenv
# import os
# import sys
# import requests
# import xml
# import pandas as pd
# from tqdm import tqdm
# from tabulate import tabulate
# from bs4 import BeautifulSoup as bs
# #from pmids_extract_solr import pmids_solr_list
# from dotenv import load_dotenv